from Screens.Screen import Screen
from Screens.FixedMenu import FixedMenu
from Screens.MessageBox import MessageBox
from Components.ActionMap import ActionMap
from Components.Sources.List import List
from Components.Sources.StaticText import StaticText
from enigma import eServiceReference, eServiceCenter
from emerlinremux import eMerlinRemux, eMerlinRemuxState
from MerlinRemuxPlayerCutEditor import MerlinRemuxPlayer
from MerlinRemuxNotificationQueueViewer import MerlinRemuxNotificationQueueViewer
from os import path as os_path
# for localized messages
from . import _

class MerlinRemuxListScreenContextMenu(FixedMenu):

	RET_DELETE_SOURCE = 0
	RET_UPDATE_VIDEODB = 1
	RET_SHOW_NOTIFICATIONS = 2

	def __init__(self, session):
		menu = [(_("back"), self.close)]
		menu.append((_("Notification Queue Viewer"), self.show_notifications))
		menu.append((_("Delete source"), self.delete_source))
		try:
			from updateVideoDB import updateVideoDB
			menu.append((_("Replace source with remuxed file in VideoDB"), self.updateVideoDB))
		except: pass		
		
		FixedMenu.__init__(self, session, _("Merlin Remux List"), menu)

	def delete_source(self):
		self.close(self.RET_DELETE_SOURCE)
	
	def updateVideoDB(self):
		self.close(self.RET_UPDATE_VIDEODB)

	def show_notifications(self):
		self.close(self.RET_SHOW_NOTIFICATIONS)

class MerlinRemuxList(List):

	def __init__(self, list):
		List.__init__(self, list = list, enableWrapAround=True, item_height=28, buildfunc=self.buildListEntry)
		self.remuxProgress_conn = eMerlinRemux.getInstance().remuxProgess.connect(self.gotRemuxProgessData)

	def buildListEntry(self, in_filename, out_filename, seek_pts, end_pts, state, id, remux_progress, audio):
		remux_state = _("State: ")
		progress_text = remux_progress
		if state == eMerlinRemuxState.running:
			remux_state = remux_state  + _("running")
		elif state == eMerlinRemuxState.finished:
			remux_state = remux_state + _("finished")
		elif state == eMerlinRemuxState.cancelled:
			remux_state = remux_state + _("cancelled")
		elif state == eMerlinRemuxState.error:
			remux_state = remux_state + _("Error")
		else:
			remux_state = remux_state + _("waiting")
		in_f = _("Filename: ") + in_filename
		out_f = _("Remux to: ") + out_filename
		return (in_f, out_f, remux_state, progress_text)

	def gotRemuxProgessData(self, data):
		index = 0
		for x in self.list:
			if x[5] == data[5]:
				self.modifyEntry(index, data)
				break
			index += 1

class MerlinRemuxListScreen(Screen):

	def __init__(self, session):
		Screen.__init__(self, session)
		self.session = session
		self.title = _("Merlin Remux List")
		self["actions"] = ActionMap(["WizardActions", "ColorActions", "MovieSelectionActions"],
		{
			"back": self.close,
			"red": self.cancel_pressed,
			"green": self.close,
			"blue": self.cleanup_pressed,
			"yellow": self.delete,
			"contextMenu": self.contextMenu_pressed,
			"ok": self.ok_pressed,
		}, -1)
		self["list"] = MerlinRemuxList(eMerlinRemux.getInstance().getTupleList())
		self["key_red"] = StaticText(_("Cancel"))
		self["key_green"] = StaticText(_("Close"))
		self["key_yellow"] = StaticText(_("Delete source"))
		self["key_blue"] = StaticText(_("Cleanup"))

	def cancel_pressed(self):
		cur = self["list"].current
		if cur and cur[4] !=  eMerlinRemuxState.finished:
			self.session.openWithCallback(self.cancelConfirmed, MessageBox, _("Do you really want to cancel?"))

	def cancelConfirmed(self, val):
		if val:
			cur = self["list"].getCurrent()
			if cur is not None:
				eMerlinRemux.getInstance().cancel(cur[5])			

	def cleanup_pressed(self):
		eMerlinRemux.getInstance().cleanUp()
		self["list"].setList(eMerlinRemux.getInstance().getTupleList())

	def ok_pressed(self):
		cur = self["list"].current
		if cur and cur[4] ==  eMerlinRemuxState.finished:
			self.session.open(MerlinRemuxPlayer, eServiceReference(eServiceReference.idGST, 0, cur[1]))

	def contextMenu_pressed(self):
		self.session.openWithCallback(self.menuCallback, MerlinRemuxListScreenContextMenu)

	def menuCallback(self, *result):
		if not len(result):
			return
		result = result[0]
		if result == MerlinRemuxListScreenContextMenu.RET_DELETE_SOURCE:
			self.delete()
		elif result == MerlinRemuxListScreenContextMenu.RET_UPDATE_VIDEODB:
			self.updateVideoDB()
		elif result == MerlinRemuxListScreenContextMenu.RET_SHOW_NOTIFICATIONS:
			self.show_notifications()
			
	def show_notifications(self):
		self.session.open(MerlinRemuxNotificationQueueViewer)

	def updateVideoDB(self):
		from updateVideoDB import updateVideoDB
		cur = self["list"].current
		if cur and cur[4] ==  eMerlinRemuxState.finished:
			updateVideoDB(self.session, cur[0], cur[1])

	def delete(self):
		cur = self["list"].current
		if cur and cur[4] ==  eMerlinRemuxState.finished:
			if os_path.exists(cur[0]):
				service = eServiceReference(eServiceReference.idDVB, 0, cur[0])
				serviceHandler = eServiceCenter.getInstance()
				info = serviceHandler.info(service)
				name = info and info.getName(service) or _("this recording")
				# simulate first
				if not eMerlinRemux.getInstance().deleteFromDisk(1,service):
					self.session.openWithCallback(self.deleteConfirmed, MessageBox, _("Do you really want to delete %s?") % (name))
				else:
					self.session.openWithCallback(self.close, MessageBox, _("You cannot delete this!"), MessageBox.TYPE_ERROR)
			else:
				self.session.open(MessageBox, _("File\n%s\ndoes not exist.") % cur[0], MessageBox.TYPE_ERROR)
		else:
			if cur:
				self.session.open(MessageBox, _("You only can delete source files with state finished."), MessageBox.TYPE_ERROR)

	def deleteConfirmed(self, confirmed):
		if not confirmed:
			return
		cur = self["list"].current
		if cur and cur[4] ==  eMerlinRemuxState.finished:
			service = eServiceReference(eServiceReference.idDVB, 0, cur[0])
			# really delete!
			if eMerlinRemux.getInstance().deleteFromDisk(0,service):
				self.session.openWithCallback(self.close, MessageBox, _("Delete failed!"), MessageBox.TYPE_ERROR)

